define([
      "dojo/_base/declare",
      "dojox/mvc/getStateful",
      "dojox/mvc/at",
      "dojo/when",
      "aps/View",
      "aps/ResourceStore",
      "aps/xhr",
      "aps/Memory",
      "aps/Output",
      "./displayError"
	],
	function (declare, getStateful, at, when, View, Store, xhr, Memory, Output, displayError) {
      var self;
      return declare(View, {
         init: function() {
            self = this;
            /* Declare the data sources */
            /* Define the data source store */
            self.repoCache = new Memory({
               data: [],
               idProperty: "id"
            });
            /* Initialize a data model to sync with widgets */
            self.licenseModel = getStateful({ "data": {
               operations: "",
               licLimit: 0,
               licUsage: 0
            }});

            var refreshLicense = function() {
               xhr.get(
                  "/aps/2/resources/" + self.tenant.aps.id + "/refreshdata")
                  .then(function() {
                     xhr.get("/aps/2/resources/" + self.tenant.aps.id)
                        .then(function (tenant) {
                           self.licenseModel.data.set("licLimit", tenant.licenseCounter.licLimit);
                           self.licenseModel.data.set("licUsage", tenant.licenseCounter.licUsage);
                        });
                  });
            };

            var refreshRepos = function() {
               when(xhr(
                  // Call the custom getRepos operation
                  "/aps/2/resources/" + self.tenant.aps.id + "/repos",
                  { method: "GET", handleAs: "json" }
                  ),
                  // When finished, refresh the Grid
                  function (repos) {
                     var data = [];
                     repos.forEach(function(repo) {
                        data.push(repo);
                     });
                     self.repoCache.setData(data);
                     self.repoGrid.refresh();
                  });
            };

            var addRepo = function() {
               var name_suffix = Math.random().toString(36).substr(2, 10);
               var payload = { "name_suffix" : name_suffix};
               when(xhr(
                  // Call the custom addRepos operation
                  "/aps/2/resources/" + self.tenant.aps.id + "/repos",
                  {
                     method: "POST",
                     handleAs: "json",
                     headers: { 'Content-Type': 'application/json' },
                     data: JSON.stringify(payload)
                  }),
                  // When finished, refresh the list of repos
                  function () {
                     refreshRepos();
                  });
            };

            var removeRepos = function() {
               var btn = this;
               /* Get confirmation from the user for the delete operation */
               if (!confirm("Are you sure you want to delete the selected Repositories?")) {
                  btn.cancel();
                  return;
               }

               var sel = self.repoGrid.get("selectionArray");
               var counter = sel.length;

               /* Clear the current messages on the screen */
               self.page.get("messageList").removeAll();

               sel.forEach(function(repoId){
                  console.log("I'm trying to delete Repo with id = [" + repoId + "]");
                  var repo = self.repoCache.get(repoId);
                  /* Remove the Repo from the APS controller DB */
                  when(xhr(
                     // Call the custom removeRepo operation
                     "/aps/2/resources/" + self.tenant.aps.id + "/repos/" + repo.name,
                     {
                        method: "DELETE"
                     }),
                     function() {
                        self.repoCache.remove(repoId);
                        console.log("Repo with id = [" + repoId + "] removed");

                        /* Remove the processed VPS from the selection */
                        sel.splice(sel.indexOf(repoId),1);
                        self.repoGrid.refresh();
                        if (--counter === 0) { btn.cancel(); }
                     },
                     /* If failure, call the error handler */
                     function(e){
                        displayError(e);
                        if (--counter === 0) { btn.cancel(); }
                     }
                  );
               });
            };



            /* Define and return widgets */
            return [
               ["aps/Panel", { title: "Service License" }, [
                  ["aps/FieldSet", [
                     ["aps/Output", {
                        label: "Licensed Operations",
                        gridSize: "md-4",
                        value: at(self.licenseModel.data, "operations")
                     }],
                     ["aps/Output", {
                        label: "Limit",
                        gridSize: "md-2",
                        value: at(self.licenseModel.data, "licLimit")
                     }],
                     ["aps/Output", {
                        label: "Used",
                        gridSize: "md-2",
                        value: at(self.licenseModel.data, "licUsage")
                     }],
                     ["aps/Button", {
                        title: "Refresh",
                        gridSize: "md-4",
                        autoBusy: false,
                        onClick: refreshLicense
                     }]
                  ]]]],
               ["aps/Panel", { title: "Repositories" }, [
                  ["aps/Grid", {
                     id: this.genId("repo_grid"),
                     store: self.repoCache,
                     selectionMode: "multiple",
                     noDataText: "No resources provisioned",
                     noEntriesFoundText: "No services meet your search criteria",
                     columns:[{
                        field: "name",
                        name: "Name"
                     },{
                        field: "url",
                        name: "URL",
                        renderCell: function(row, URL) {
                           var link = "<a href=" + URL + " target='_blank'>" + URL + "</a>";
                           return new Output({
                              content: link
                           });
                        }
                     }]
                  },[
                     ["aps/Toolbar", [
                        ["aps/ToolbarButton", {
                           iconClass:"fa-refresh",
                           label: "Refresh",
                           type: "primary",
                           autoBusy: false,
                           onClick: refreshRepos
                        }],
                        ["aps/ToolbarButton", {
                           iconClass:"fa-plus",
                           label: "Add",
                           type: "success",
                           autoBusy: false,
                           onClick: addRepo
                        }],
                        ["aps/ToolbarButton", {
                           iconClass:"fa-trash",
                           label: "Remove",
                           type: "warning",
                           requireItems: true,
                           autoBusy: false,
                           onClick: removeRepos
                        }]
                     ]]
            ]]]]];
         }, // End of Init

         onContext: function(context) {
            self.page = this.byId("apsPageContainer");
            self.tenant = context.vars.tenant;
            self.licenseApsId = self.tenant.license.aps.id;
            xhr.get("/aps/2/resources/" + self.licenseApsId)
            .then(function(license) {
               self.licenseModel.data.set("operations", license.scopes.toString());
               self.licenseModel.data.set("licLimit", self.tenant.licenseCounter.licLimit);
               self.licenseModel.data.set("licUsage", self.tenant.licenseCounter.licUsage);

               self.repoGrid = self.byId("repo_grid");
               self.repoGrid.refresh();
               aps.apsc.hideLoading();
            });
         }  // End of onContext
		
      });   // End of Declare
});         // End of Define

